@@ -35,6 +35,7 @@ GltfSettings = collections.namedtuple('GltfSettings', (
     'skip_axis_conversion',
     'no_srgb',
     'textures',
+    'legacy_materials',
 ))
 GltfSettings.__new__.__defaults__ = (
     'builtin', # physics engine
@@ -42,6 +43,7 @@ GltfSettings.__new__.__defaults__ = (
     False, # skip_axis_conversion
     False, # do not load textures as sRGB
     'ref', # reference external textures
+    False, # use PBR materials
 )
 
 
@@ -55,6 +57,15 @@ class Converter():
         5125: GeomEnums.NT_uint32,
         5126: GeomEnums.NT_float32,
     }
+    _COMPONENT_SIZE_MAP = {
+        5120: 1,
+        5121: 1,
+        5122: 2,
+        5123: 2,
+        5124: 4,
+        5125: 4,
+        5126: 4,
+    }
     _COMPONENT_NUM_MAP = {
         'MAT4': 16,
         'VEC4': 4,
@@ -241,11 +252,6 @@ class Converter():
                     if cvsmap:
                         self.combine_mesh_morphs(mesh, meshid, cvsmap)
 
-            if 'skin' in gltf_node and not 'mesh' in gltf_node:
-                print(
-                    "Warning: node {} has a skin but no mesh"
-                    .format(primitiveid)
-                )
             if 'camera' in gltf_node:
                 camid = gltf_node['camera']
                 cam = self.cameras[camid]
@@ -579,59 +585,78 @@ class Converter():
         normal_fallback = {'index': '__normal-fallback', 'texCoord': 0}
         texinfos = []
 
-        if 'extensions' in gltf_mat and 'BP_materials_legacy' in gltf_mat['extensions']:
-            matsettings = gltf_mat['extensions']['BP_materials_legacy']['bpLegacy']
-            pmat.set_shininess(matsettings['shininessFactor'])
-            pmat.set_ambient(LColor(*matsettings['ambientFactor']))
+        if self.settings.legacy_materials:
+            if 'pbrMetallicRoughness' in gltf_mat:
+                pbrsettings = gltf_mat['pbrMetallicRoughness']
 
-            if 'diffuseTexture' in matsettings:
-                texinfo = matsettings['diffuseTexture']
-                texinfos.append(texinfo)
-                if matsettings['diffuseTextureSrgb'] and texinfo['index'] in self.textures:
-                    self.make_texture_srgb(self.textures[texinfo['index']])
-            else:
-                pmat.set_diffuse(LColor(*matsettings['diffuseFactor']))
+                pmat.set_diffuse(LColor(*pbrsettings.get('baseColorFactor', [1.0, 1.0, 1.0, 1.0])))
+                texinfos.append(pbrsettings.get('baseColorTexture', pbr_fallback))
+                if texinfos[-1]['index'] in self.textures:
+                    self.make_texture_srgb(self.textures[texinfos[-1]['index']])
 
-            if 'emissionTexture' in matsettings:
-                texinfo = matsettings['emissionTexture']
-                texinfos.append(texinfo)
-                if matsettings['emissionTextureSrgb'] and texinfo['index'] in self.textures:
-                    self.make_texture_srgb(self.textures[texinfo['index']])
-            else:
-                pmat.set_emission(LColor(*matsettings['emissionFactor']))
+            texinfos.append(gltf_mat.get('normalTexture', normal_fallback))
+            texinfos[-1]['mode'] = TextureStage.M_normal
+        else:
+            if 'extensions' in gltf_mat and 'BP_materials_legacy' in gltf_mat['extensions']:
+                matsettings = gltf_mat['extensions']['BP_materials_legacy']['bpLegacy']
+                pmat.set_shininess(matsettings['shininessFactor'])
+                pmat.set_ambient(LColor(*matsettings['ambientFactor']))
 
-            if 'specularTexture' in matsettings:
-                texinfo = matsettings['specularTexture']
-                texinfos.append(texinfo)
-                if matsettings['specularTextureSrgb'] and texinfo['index'] in self.textures:
-                    self.make_texture_srgb(self.textures[texinfo['index']])
-            else:
-                pmat.set_specular(LColor(*matsettings['specularFactor']))
-        elif 'pbrMetallicRoughness' in gltf_mat:
-            pbrsettings = gltf_mat['pbrMetallicRoughness']
+                if 'diffuseTexture' in matsettings:
+                    texinfo = matsettings['diffuseTexture']
+                    texinfos.append(texinfo)
+                    if matsettings['diffuseTextureSrgb'] and texinfo['index'] in self.textures:
+                        self.make_texture_srgb(self.textures[texinfo['index']])
+                else:
+                    pmat.set_diffuse(LColor(*matsettings['diffuseFactor']))
 
-            pmat.set_base_color(LColor(*pbrsettings.get('baseColorFactor', [1.0, 1.0, 1.0, 1.0])))
-            texinfos.append(pbrsettings.get('baseColorTexture', pbr_fallback))
+                if 'emissionTexture' in matsettings:
+                    texinfo = matsettings['emissionTexture']
+                    texinfos.append(texinfo)
+                    if matsettings['emissionTextureSrgb'] and texinfo['index'] in self.textures:
+                        self.make_texture_srgb(self.textures[texinfo['index']])
+                else:
+                    pmat.set_emission(LColor(*matsettings['emissionFactor']))
+
+                if 'specularTexture' in matsettings:
+                    texinfo = matsettings['specularTexture']
+                    texinfos.append(texinfo)
+                    if matsettings['specularTextureSrgb'] and texinfo['index'] in self.textures:
+                        self.make_texture_srgb(self.textures[texinfo['index']])
+                else:
+                    pmat.set_specular(LColor(*matsettings['specularFactor']))
+            elif 'pbrMetallicRoughness' in gltf_mat:
+                pbrsettings = gltf_mat['pbrMetallicRoughness']
+
+                pmat.set_base_color(LColor(*pbrsettings.get('baseColorFactor', [1.0, 1.0, 1.0, 1.0])))
+                texinfos.append(pbrsettings.get('baseColorTexture', pbr_fallback))
+                if texinfos[-1]['index'] in self.textures:
+                    self.make_texture_srgb(self.textures[texinfos[-1]['index']])
+
+                pmat.set_metallic(pbrsettings.get('metallicFactor', 1.0))
+                pmat.set_roughness(pbrsettings.get('roughnessFactor', 1.0))
+                texinfos.append(pbrsettings.get('metallicRoughnessTexture', pbr_fallback))
+                texinfos[-1]['mode'] = TextureStage.M_selector # Unused, but means FFP and auto shader will ignore
+
+            # Normal map
+            texinfos.append(gltf_mat.get('normalTexture', normal_fallback))
+            texinfos[-1]['mode'] = TextureStage.M_normal
+
+            # Emission map
+            pmat.set_emission(LColor(*gltf_mat.get('emissiveFactor', [0.0, 0.0, 0.0]), w=0.0))
+            texinfos.append(gltf_mat.get('emissiveTexture', emission_fallback))
+            texinfos[-1]['mode'] = TextureStage.M_add
             if texinfos[-1]['index'] in self.textures:
                 self.make_texture_srgb(self.textures[texinfos[-1]['index']])
 
-            pmat.set_metallic(pbrsettings.get('metallicFactor', 1.0))
-            pmat.set_roughness(pbrsettings.get('roughnessFactor', 1.0))
-            texinfos.append(pbrsettings.get('metallicRoughnessTexture', pbr_fallback))
-
-        # Normal map
-        texinfos.append(gltf_mat.get('normalTexture', normal_fallback))
-
-        #Emission map
-        pmat.set_emission(LColor(*gltf_mat.get('emissiveFactor', [0.0, 0.0, 0.0]), w=0.0))
-        texinfos.append(gltf_mat.get('emissiveTexture', emission_fallback))
-        if texinfos[-1]['index'] in self.textures:
-            self.make_texture_srgb(self.textures[texinfos[-1]['index']])
-
-        pmat.set_twoside(gltf_mat.get('doubleSided', False))
+        double_sided = gltf_mat.get('doubleSided', False)
+        pmat.set_twoside(double_sided)
 
         state = state.set_attrib(MaterialAttrib.make(pmat))
 
+        if double_sided:
+            state = state.set_attrib(CullFaceAttrib.make(CullFaceAttrib.MCullNone))
+
         # Setup textures
         tex_attrib = TextureAttrib.make()
         for i, texinfo in enumerate(texinfos):
@@ -643,6 +668,7 @@ class Converter():
             texstage = TextureStage(str(i))
             texstage.set_sort(i)
             texstage.set_texcoord_name(InternalName.get_texcoord_name(str(texinfo.get('texCoord', 0))))
+            texstage.set_mode(texinfo.get('mode', TextureStage.M_modulate))
             tex_attrib = tex_attrib.add_on_stage(texstage, texdata)
 
         state = state.set_attrib(tex_attrib)
@@ -754,6 +780,7 @@ class Converter():
                     internal_name = InternalName.make(attrib_name)
                 num_components = self._COMPONENT_NUM_MAP[acc['type']]
                 numeric_type = self._COMPONENT_TYPE_MAP[acc['componentType']]
+                numeric_size = self._COMPONENT_SIZE_MAP[acc['componentType']]
                 content = self._ATTRIB_CONTENT_MAP.get(attrib_name, GeomEnums.C_other)
 
                 if '_target' in acc:
@@ -771,7 +798,7 @@ class Converter():
                         buffview['buffer'],
                         acc.get('byteOffset', 0) + buffview.get('byteOffset', 0),
                         acc['count'],
-                        buffview.get('byteStride', 4 * num_components)
+                        buffview.get('byteStride', numeric_size * num_components)
                     ))
 
             if is_interleaved:
@@ -895,13 +922,19 @@ class Converter():
                 "Warning: mesh {} has a primitive with no material, using an empty RenderState"
                 .format(geom_node.name)
             )
-            mat = RenderState.make_empty()
+            pmat = Material('fallback material')
+            matattrib = MaterialAttrib.make(pmat)
+            texattrib = TextureAttrib.make(self.textures.get('__pbr-fallback'))
+            mat = RenderState.make(matattrib, texattrib)
         elif matid not in self.mat_states:
             print(
                 "Warning: material with name {} has no associated mat state, using an empty RenderState"
                 .format(matid)
             )
-            mat = RenderState.make_empty()
+            pmat = Material('fallback material')
+            matattrib = MaterialAttrib.make(pmat)
+            texattrib = TextureAttrib.make(self.textures.get('__pbr-fallback'))
+            mat = RenderState.make(matattrib, texattrib)
         else:
             mat = self.mat_states[gltf_primitive['material']]
             self.mat_mesh_map[gltf_primitive['material']].append((geom_node.name, primitiveid))
